# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    fabulous.rotating_cube
    ~~~~~~~~~~~~~~~~~~~~~~

    Command for animating a wireframe rotating cube in the terminal.

"""

from __future__ import with_statement
from __future__ import division

import sys
import time
from math import cos, sin, pi

from fabulous import color, utils


class Frame(object):
    """Canvas object for drawing a frame to be printed
    """

    def __enter__(self):
        self.width = utils.term.width
        self.height = utils.term.height * 2
        self.canvas = [[' ' for x in range(self.width)]
                       for y in range(self.height // 2)]
        return self

    def __exit__(self, type_, value, traceback):
        sys.stdout.write(self.render())
        sys.stdout.flush()

    def __setitem__(self, p, c):
        (x, y) = p
        self.canvas[int(y // 2)][int(x)] = c

    def line(self, x0, y0, x1, y1, c='*'):
        r"""Draws a line

        Who would have thought this would be so complicated?  Thanks
        again Wikipedia_ <3

        .. _Wikipedia: http://en.wikipedia.org/wiki/Bresenham's_line_algorithm
        """
        steep = abs(y1 - y0) > abs(x1 - x0)
        if steep:
            (x0, y0) = (y0, x0)
            (x1, y1) = (y1, x1)
        if x0 > x1:
            (x0, x1) = (x1, x0)
            (y0, y1) = (y1, y0)
        deltax = x1 - x0
        deltay = abs(y1 - y0)
        error = deltax / 2
        y = y0
        if y0 < y1:
            ystep = 1
        else:
            ystep = -1
        for x in range(x0, x1 - 1):
            if steep:
                self[y, x] = c
            else:
                self[x, y] = c
            error = error - deltay
            if error < 0:
                y = y + ystep
                error = error + deltax

    def render(self):
        return "\n".join(["".join(line) for line in self.canvas])


def rotating_cube(degree_change=3, frame_rate=3):
    """Rotating cube program

    How it works:

      1. Create two imaginary ellipses
      2. Sized to fit in the top third and bottom third of screen
      3. Create four imaginary points on each ellipse
      4. Make those points the top and bottom corners of your cube
      5. Connect the lines and render
      6. Rotate the points on the ellipses and repeat

    """
    degrees = 0
    while True:
        t1 = time.time()

        with Frame() as frame:
            oval_width = frame.width
            oval_height = frame.height / 3.0
            cube_height = int(oval_height * 2)

            (p1_x, p1_y) = ellipse_point(degrees, oval_width, oval_height)
            (p2_x, p2_y) = ellipse_point(degrees + 90, oval_width, oval_height)
            (p3_x, p3_y) = ellipse_point(degrees + 180, oval_width, oval_height)
            (p4_x, p4_y) = ellipse_point(degrees + 270, oval_width, oval_height)
            degrees = (degrees + degree_change) % 360

            # connect square thing at top
            frame.line(p1_x, p1_y, p2_x, p2_y)
            frame.line(p2_x, p2_y, p3_x, p3_y)
            frame.line(p3_x, p3_y, p4_x, p4_y)
            frame.line(p4_x, p4_y, p1_x, p1_y)

            # connect top to bottom
            frame.line(p1_x, p1_y, p1_x, p1_y + cube_height)
            frame.line(p2_x, p2_y, p2_x, p2_y + cube_height)
            frame.line(p3_x, p3_y, p3_x, p3_y + cube_height)
            frame.line(p4_x, p4_y, p4_x, p4_y + cube_height)

            # connect square thing at bottom
            frame.line(p1_x, p1_y + cube_height, p2_x, p2_y + cube_height)
            frame.line(p2_x, p2_y + cube_height, p3_x, p3_y + cube_height)
            frame.line(p3_x, p3_y + cube_height, p4_x, p4_y + cube_height)
            frame.line(p4_x, p4_y + cube_height, p1_x, p1_y + cube_height)

        elapsed = (time.time() - t1)
        time.sleep(abs(1.0 / frame_rate - elapsed))


def ellipse_point(degrees, width, height):
    width -= 1
    height -= 1
    radians = degrees * (pi / 180.0)
    x = width/2.0 * cos(1) * sin(radians) - width/2.0 * sin(1) * cos(radians)
    y = height/2.0 * sin(1) * sin(radians) + height/2.0 * cos(1) * cos(radians)
    x = int(x + width/2.0)
    y = int(y + height/2.0)
    return (x, y)


def main():
    try:
        rotating_cube()
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This is a long timed progress bar. Don't expect it to finish any 
time soon. It updates every 0 to 10 seconds. Send it a keyboard 
interupt (Ctrl-C) to stop."""

from fabulous.widget import TimedProgressBar
import time
import random

print __doc__

p = TimedProgressBar('bar')
p.precision=10000
n = 100000
for i in range(n):
    p.update(float(i)/n *100, 'update '+str(i))
    time.sleep(random.random()* 10)



#!/usr/bin/env python
#
# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""I am *not* responsible if this causes severe physical or mental injury
"""

from fabulous.term import display, stdout, Magic
        

try:
    while True:
        for c in Magic.COLORS.iterkeys():
            stdout.write(display(bg=c))
            stdout.move('down',4)
except KeyboardInterrupt:
    stdout.write(display('default'))
    print
    print "Interupt from keyboard. Exiting."



# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    fabulous.utils
    ~~~~~~~~~~~~~~

    Miscellaneous utilities for Fabulous.

"""

import os
import sys
import fcntl
import struct
import termios
import textwrap
import functools

from fabulous import grapefruit


def memoize(function):
    """A very simple memoize decorator to optimize pure-ish functions

    Don't use this unless you've examined the code and see the
    potential risks.
    """
    cache = {}
    @functools.wraps(function)
    def _memoize(*args):
        if args in cache:
            return cache[args]
        result = function(*args)
        cache[args] = result
        return result
    return function


class TerminalInfo(object):
    """Quick and easy access to some terminal information

    I'll tell you the terminal width/height and it's background color.

    You don't need to use me directly.  Just access the global
    :data:`term` instance::

        >>> assert term.width > 0
        >>> assert term.height > 0

    It's important to know the background color when rendering PNG
    images with semi-transparency.  Because there's no way to detect
    this, black will be the default::

        >>> term.bgcolor
        (0.0, 0.0, 0.0, 1.0)
        >>> from fabulous import grapefruit
        >>> isinstance(term.bgcolor, grapefruit.Color)
        True

    If you use a white terminal, you'll need to manually change this::

        >>> term.bgcolor = 'white'
        >>> term.bgcolor
        (1.0, 1.0, 1.0, 1.0)
        >>> term.bgcolor = grapefruit.Color.NewFromRgb(0.0, 0.0, 0.0, 1.0)
        >>> term.bgcolor
        (0.0, 0.0, 0.0, 1.0)

    """

    def __init__(self, bgcolor='black'):
        self.bgcolor = bgcolor

    @property
    def termfd(self):
        """Returns file descriptor number of terminal

        This will look at all three standard i/o file descriptors and
        return whichever one is actually a TTY in case you're
        redirecting i/o through pipes.
        """
        for fd in (2, 1, 0):
            if os.isatty(fd):
                return fd
        raise Exception("No TTY could be found")

    @property
    def dimensions(self):
        """Returns terminal dimensions

        Don't save this information for long periods of time because
        the user might resize their terminal.

        :return: Returns ``(width, height)``.  If there's no terminal
                 to be found, we'll just return ``(79, 40)``.
        """
        try:
            call = fcntl.ioctl(self.termfd, termios.TIOCGWINSZ, "\000" * 8)
        except IOError:
            return (79, 40)
        else:
            height, width = struct.unpack("hhhh", call)[:2]
            return (width, height)

    @property
    def width(self):
        """Returns width of terminal in characters
        """
        return self.dimensions[0]

    @property
    def height(self):
        """Returns height of terminal in lines
        """
        return self.dimensions[1]

    def _get_bgcolor(self):
        return self._bgcolor

    def _set_bgcolor(self, color):
        if isinstance(color, grapefruit.Color):
            self._bgcolor = color
        else:
            self._bgcolor = grapefruit.Color.NewFromHtml(color)

    bgcolor = property(_get_bgcolor, _set_bgcolor)


term = TerminalInfo()


def pil_check():
    """Check for PIL library, printing friendly error if not found

    We need PIL for the :mod:`fabulous.text` and :mod:`fabulous.image`
    modules to work.  Because PIL can be very tricky to install, it's
    not listed in the ``setup.py`` requirements list.
    """
    try:
        import PIL
    except ImportError:
        raise ImportError("Please install PIL to use this feature: "
                          "https://pillow.readthedocs.io/en/latest"
                          "/installation.html")



# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Input and output functions
"""

import sys
import os
import os.path
import term
from term import stdout, stderr, display

__all__ = ["input_object","query","file_chooser"]

# this constant is here because float() and int() give error messages
# that would confuse most sane users.

ERROR_MESSAGE = ( display('bright','red') + 'Error: ' + display('default') +
                 '%s' + '\a' + os.linesep )
NICE_INPUT_ERRORS = {
    float: "The input ('%s') must be a number",
    int: "The input ('%s') must be an integer (-1, 0, 1, 2, etc.)"
}

DEFAULT_INPUT_ERRORS = "Bad input (%s)"

def input_object(prompt_text, cast = None, default = None,
                 prompt_ext = ': ', castarg = [], castkwarg = {}):
    """Gets input from the command line and validates it.
    
    prompt_text
        A string. Used to prompt the user. Do not include a trailing
        space.
        
    prompt_ext
        Added on to the prompt at the end. At the moment this must not
        include any control stuff because it is send directly to
        raw_input
        
    cast
        This can be any callable object (class, function, type, etc). It
        simply calls the cast with the given arguements and returns the 
        result. If a ValueError is raised, it
        will output an error message and prompt the user again.

        Because some builtin python objects don't do casting in the way
        that we might like you can easily write a wrapper function that
        looks and the input and returns the appropriate object or exception.
        Look in the cast submodule for examples.
        
        If cast is None, then it will do nothing (and you will have a string)
        
    default
        function returns this value if the user types nothing in. This is
        can be used to cancel the input so-to-speek
        
    castarg, castkwarg
        list and dictionary. Extra arguments passed on to the cast.
    """
    while True:
        stdout.write(prompt_text)
        value = stdout.raw_input(prompt_ext)
        if value == '': return default
        try:
            if cast != None: value = cast(value, *castarg, **castkwarg)
        except ValueError as details:
            if cast in NICE_INPUT_ERRORS: # see comment above this constant
                stderr.write(ERROR_MESSAGE % (NICE_INPUT_ERRORS[cast] % details))
            else: stderr.write(ERROR_MESSAGE % (DEFAULT_INPUT_ERRORS % str(details)))
            continue
        return value

def query(question, values, default=None, list_values = False, ignorecase = True ):
    """Preset a few options
    
    The question argument is a string, nothing magical.
    
    The values argument accepts input in two different forms. The simpler form
    (a tuple with strings) looks like:
    
        .. code-block:: python
        
            ('Male','Female')
    
    And it will pop up a question asking the user for a gender and requiring
    the user to enter either 'male' or 'female' (case doesn't matter unless
    you set the third arguement to false).
    The other form is something like:
    
        .. code-block:: python
        
            ({'values':('Male','M'),'fg':'cyan'},
            {'values':('Female','F'),'fg':'magenta'})
    
    This will pop up a question with Male/Female (each with appropriate
    colouring). Additionally, if the user types in just 'M', it will be
    treated as if 'Male' was typed in. The first item in the 'values' tuple
    is treated as default and is the one that is returned by the function
    if the user chooses one in that group.
    In addition the function can handle non-string objects quite fine. It
    simple displays the output object.__str__() and compares the user's input
    against that. So the the code
    
        .. code-block:: python
        
            query("Python rocks? ",(True, False))
    
    will return a bool (True) when the user types in the string 'True' (Of
    course there isn't any other reasonable answer than True anyways :P)
    
    ``default`` is the value function returns if the user types nothing in. This is
    can be used to cancel the input so-to-speek
    
    Using list_values = False will display a list, with descriptions printed out
    from the 'desc' keyword
    """
    values = list(values)
    for i in range(len(values)):
        if not isinstance(values[i], dict):
            values[i] = {'values': [values[i]]}
    try:
        import readline, rlcomplete
        wordlist = [ str(v) for value in values
                    for v in value['values']]
        completer = rlcomplete.ListCompleter(wordlist, ignorecase)
        readline.parse_and_bind("tab: complete")
        readline.set_completer(completer.complete)
    except ImportError:
        pass
    valuelist = []
    for item in values:
        entry = ( display('bright', item.get('fg'), item.get('bg')) +
            str(item['values'][0]) + display(['default']) )
        if str(item['values'][0]) == str(default): entry = '['+entry+']'
        if list_values: entry += ' : ' + item['desc']
        valuelist.append(entry)
    if list_values: question += os.linesep + os.linesep.join(valuelist) + os.linesep
    else: question += ' (' + '/'.join(valuelist) + ')'
    return input_object(question, cast = query_cast, default=default,
                 castarg=[values,ignorecase])

def query_cast(value, answers, ignorecase = False):
    """A cast function for query
    
    Answers should look something like it does in query
    """
    if ignorecase: value = value.lower()
    for item in answers:
        for a in item['values']:
            if ignorecase and (value == str(a).lower()):
                return item['values'][0]
            elif value == a:
                return item['values'][0]
    raise ValueError("Response '%s' not understood, please try again." % value)

def file_chooser(prompt_text = "Enter File: ", default=None, filearg=[], filekwarg={}):
    """A simple tool to get a file from the user. Takes keyworded arguemnts
    and passes them to open().
    
    If the user enters nothing the function will return the ``default`` value.
    Otherwise it continues to prompt the user until it get's a decent response.
    
    filekwarg may contain arguements passed on to ``open()``.
    """
    try:
        import readline, rlcomplete
        completer = rlcomplete.PathCompleter()
        readline.set_completer_delims(completer.delims)
        readline.parse_and_bind("tab: complete")
        readline.set_completer(completer.complete)
    except ImportError:
        pass
    while True:
        f = raw_input(prompt_text)
        if f == '': return default
        f = os.path.expanduser(f)
        if len(f) != 0 and f[0] == os.path.sep:
            f = os.path.abspath(f)
        try:
            return open(f, *filearg, **filekwarg)
        except IOError as e:
            stderr.write(ERROR_MESSAGE % ("unable to open %s : %s" % (f, e)))

if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python
#
# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from fabulous.term import display, stdout, Magic

stdout.write(display('default'))
stdout.write("default text\n")

stdout.write("regular foreground test:\n")
for color in Magic.COLORS.iterkeys():
    stdout.write(display(fg=color))
    stdout.write("    " + color + '\n')

stdout.write(display('default'))

stdout.write("regular background test:\n")
for color in Magic.COLORS.iterkeys():
    stdout.write(display(bg=color))
    stdout.write("    " + color + '\n')

stdout.write(display('default'))

stdout.write("bright foreground test:\n")
stdout.write(display('bright'))
for color in Magic.COLORS.iterkeys():
    stdout.write(display(fg=color))
    stdout.write("    " + color + '\n')

stdout.write(display('default'))

stdout.write("dim foreground test:\n")
stdout.write(display('dim'))
for color in Magic.COLORS.iterkeys():
    stdout.write(display(fg=color))
    stdout.write("    " + color + '\n')

stdout.write(display('bright','red'))
stdout.write("bright red\n")
stdout.write(display("default"))



#!/usr/bin/env python
#
# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from fabulous.widget import ProgressBar
import time

p = ProgressBar('bar')
p.update(0)
time.sleep(.2)
p.update(20, 'safe epam and eggs')
time.sleep(.2)
p.update(50, None)
time.sleep(.2)
p.update(40)
time.sleep(.2)
p.set_title('baz')
p.update(20, 'spam and eggs')
time.sleep(.2)
p.update(70)
time.sleep(.2)
p.update(80)
time.sleep(.2)
p.update(100)



#!/usr/bin/env python
#
# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from fabulous import rlcomplete

class TestControl(unittest.TestCase):

    def test_ListCompleter(self):
        c = rlcomplete.ListCompleter(['foo','bar','baz'], True)
        self.failUnlessEqual(c.completelist('a'), [])
        self.failUnlessEqual(c.completelist('B'), ['bar','baz'])
        self.failUnlessEqual(c.completelist('f'), ['foo'])
        c = rlcomplete.ListCompleter(['foo','bar','baz'], False)
        self.failUnlessEqual(c.completelist('B'), [])
        self.failUnlessEqual(c.completelist('b'), ['bar','baz'])

    def test_PathCompleter(self):
        #c = rlcomplete.PathCompleter()
        #self.failUnlessEqual(c.completelist('m'), ['manual/'])
        pass
        
if __name__ == '__main__':
    unittest.main()

#Desktop/    bin/  lib/    personal/  src/  workspace/
#Templates/  doc/  media/  share/     tmp/  www/



#!/usr/bin/env python
#
# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Outputs something like:
spaand eggs
george
"""

from fabulous.term import stdout

stdout.write('spam')
stdout.move('left')
stdout.write('and')
stdout.move('right')
stdout.write('eggs')
stdout.move('left',3)
stdout.move('down')
stdout.write('george')
print # trailing newline



# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    fabulous.logs
    ~~~~~~~~~~~~~

    Utilities for transient logging.

    This is very useful tool for monitoring what your Python scripts are doing.
    It allows you to have full verbosity without drowning out important error
    messages:

    .. code-block:: python

        import time, logging
        from fabulous import logs
        logs.basicConfig(level='WARNING')

        for n in range(20):
            logging.debug("verbose stuff you don't care about")
            time.sleep(0.1)
        logging.warning("something bad happened!")
        for n in range(20):
            logging.debug("verbose stuff you don't care about")
            time.sleep(0.1)

"""

import sys
import logging

from fabulous import utils


class TransientStreamHandler(logging.StreamHandler):
    """Standard Python logging Handler for Transient Console Logging

    Logging transiently means that verbose logging messages like DEBUG
    will only appear on the last line of your terminal for a short
    period of time and important messages like WARNING will scroll
    like normal text.

    This allows you to log lots of messages without the important
    stuff getting drowned out.

    This module integrates with the standard Python logging module.
    """

    def __init__(self, strm=sys.stderr, level=logging.WARNING):
        logging.StreamHandler.__init__(self, strm)
        if isinstance(level, int):
            self.levelno = level
        else:
            self.levelno = logging._levelNames[level]
        self.need_cr = False
        self.last = ""
        self.parent = logging.StreamHandler

    def close(self):
        if self.need_cr:
            self.stream.write("\n")
            self.need_cr = False
        self.parent.close(self)

    def write(self, data):
        if self.need_cr:
            width = max(min(utils.term.width, len(self.last)), len(data))
            fmt = "\r%-" + str(width) + "s\n" + self.last
        else:
            fmt = "%s\n"
        try:
            self.stream.write(fmt % (data))
        except UnicodeError:
            self.stream.write(fmt % (data.encode("UTF-8")))

    def transient_write(self, data):
        if self.need_cr:
            self.stream.write('\r')
        else:
            self.need_cr = True
        width = utils.term.width
        for line in data.rstrip().split('\n'):
            if line:
                if len(line) > width:
                    line = line[:width - 3] + '...'
                line_width = max(min(width, len(self.last)), len(line))
                fmt = "%-" + str(line_width) + "s"
                self.last = line
                try:
                    self.stream.write(fmt % (line))
                except UnicodeError:
                    self.stream.write(fmt % (line.encode("UTF-8")))
            else:
                self.stream.write('\r')
                self.stream.flush()

    def emit(self, record):
        try:
            msg = self.format(record)
            if record.levelno >= self.levelno:
                self.write(msg)
            else:
                self.transient_write(msg)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


def basicConfig(level=logging.WARNING, transient_level=logging.NOTSET):
    """Shortcut for setting up transient logging

    I am a replica of ``logging.basicConfig`` which installs a
    transient logging handler to stderr.
    """
    fmt = "%(asctime)s [%(levelname)s] [%(name)s:%(lineno)d] %(message)s"
    logging.root.setLevel(transient_level)  # <--- IMPORTANT
    hand = TransientStreamHandler(level=level)
    hand.setFormatter(logging.Formatter(fmt))
    logging.root.addHandler(hand)



# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# http://packages.python.org/distribute/setuptools.html
# http://diveintopython3.org/packaging.html
# http://wiki.python.org/moin/CheeseShopTutorial
# http://pypi.python.org/pypi?:action=list_classifiers

import os

try:
    import setuptools
except ImportError:
    import ez_setup
    ez_setup.use_setuptools(version='0.6c11')
    import setuptools

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

version = __import__('fabulous').__version__

setuptools.setup(
    name                 = 'fabulous',
    version              = version,
    url                  = 'https://jart.github.io/fabulous',
    author               = 'Justine Tunney',
    author_email         = 'jtunney@gmail.com',
    description          = 'Makes your terminal output totally fabulous',
    download_url         = ('https://github.com/jart/fabulous/releases'
                            '/download/' + version +
                            '/fabulous-' + version + '.tar.gz'),
    long_description     = read('README.rst'),
    license              = 'Apache 2.0 / OFL',
    packages             = ['fabulous', 'fabulous.experimental'],
    zip_safe             = False,
    include_package_data = True,
    entry_points = {
        'console_scripts': [
            'fabulous-demo = fabulous.demo:main',
            'fabulous-gotham = fabulous.gotham:main',
            'fabulous-image = fabulous.image:main',
            'fabulous-rotatingcube = fabulous.rotating_cube:main',
            'fabulous-text = fabulous.text:main',
        ],
    },
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: Apache Software License",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Programming Language :: C",
        "Programming Language :: Python",
        "Topic :: Utilities",
        "Topic :: Artistic Software",
        "Topic :: System :: Logging",
        "Topic :: Multimedia :: Graphics"
    ],
)



# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    fabulous.gotham
    ~~~~~~~~~~~~~~~

    This is a gimmick feature that generates silly gothic poetry.

    This uses a simple mad lib algorithm. It has no concept of meter or rhyme.
    If you want a *proper* poetry generator, check out poemy2_ which uses markov
    chains and isledict. It's written by the same author as Fabulous.

    This module can be run as a command line tool::

        jart@compy:~$ fabulous-gotham
        jart@compy:~$ python -m fabulous.gotham

    .. _poemy2: https://github.com/jart/poemy2

"""

from __future__ import print_function

import sys
import random
import itertools

try:
  next
except NameError:
  next = lambda x: x.next()


them = ['angels', 'mourners', 'shadows', 'storm clouds', 'memories', 'condemned',
        'hand of Heaven', 'stroke of death', 'damned', 'witches', 'corpses']
them_verb = ['follow', 'hover close', 'approach', 'loom', 'taunt',
             'laugh', 'surround', 'compell', 'scour']
adj = ['cold', 'dead', 'dark', 'frozen', 'angry', 'ghastly', 'unholy', 'cunning', 'deep',
       'morose', 'maligned', 'rotting', 'sickly']
me_part = ['neck', 'heart', 'head', 'eyes', 'soul', 'blood', 'essence', 'wisdom']
feeling = ['pain', 'horror', 'frenzy', 'agony', 'numbness', 'fear', 'love',
           'terror', 'madness', 'torment', 'bitterness', 'misery']
angst = ['care', 'understand', 'question']
me_verb = ['flee', 'dance', 'flail madly', 'fall limply', 'hang my head', 'try to run',
           'cry out', 'call your name', 'beg forgiveness', 'bleed', 'tremble', 'hear']
action = ['sever', 'crush', 'mutilate', 'slay', 'wound', 'smite', 'drip',
          'melt', 'cast', 'mourn', 'avenge']
place = ['the witching hour', 'the gates of hell', 'the door', 'the path', 'death',
         'my doom', 'oblivion', 'the end of life', 'Hell', 'nothingness', 'purgatory',
         'void', 'earth', 'tomb', 'broken ground', 'barren land', 'swirling dust']


def lorem_gotham():
    """Cheesy Gothic Poetry Generator

    Uses Python generators to yield eternal angst.

    When you need to generate random verbiage to test your code or
    typographic design, let's face it... Lorem Ipsum and "the quick
    brown fox" are old and boring!

    What you need is something with *flavor*, the kind of thing a
    depressed teenager with a lot of black makeup would write.
    """
    w = lambda l: l[random.randrange(len(l))]
    er = lambda w: w[:-1]+'ier' if w.endswith('y') else (w+'r' if w.endswith('e') else w+'er')
    s = lambda w: w+'s'
    punc = lambda c, *l: " ".join(l)+c
    sentence = lambda *l: lambda: " ".join(l)
    pick = lambda *l: (l[random.randrange(len(l))])()
    while True:
        yield pick(
            sentence('the',w(adj),w(them),'and the',w(them),w(them_verb)),
            sentence('delivering me to',w(place)),
            sentence('they',w(action),'my',w(me_part),'and',w(me_verb),'with all my',w(feeling)),
            sentence('in the',w(place),'my',w(feeling),'shall',w(me_verb)),
            sentence(punc(',', er(w(adj)),'than the a petty',w(feeling))),
            sentence(er(w(adj)),'than',w(them),'in',w(place)),
            sentence(punc('!','oh my',w(me_part)),punc('!','the',w(feeling))),
            sentence('no one',s(w(angst)),'why the',w(them),w(them_verb + me_verb)))


def lorem_gotham_title():
    """Names your poem
    """
    w = lambda l: l[random.randrange(len(l))]
    sentence = lambda *l: lambda: " ".join(l)
    pick = lambda *l: (l[random.randrange(len(l))])()
    return pick(
        sentence('why i',w(me_verb)),
        sentence(w(place)),
        sentence('a',w(adj),w(adj),w(place)),
        sentence('the',w(them)))


def main():
    """I provide a command-line interface for this module
    """
    print()
    print("-~*~--~*~--~*~--~*~--~*~--~*~--~*~--~*~--~*~--~*~-")
    print(lorem_gotham_title().center(50))
    print("-~*~--~*~--~*~--~*~--~*~--~*~--~*~--~*~--~*~--~*~-")
    print()
    poem = lorem_gotham()
    for n in range(16):
        if n in (4, 8, 12):
            print()
        print(next(poem))
    print()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from fabulous.term import display, Magic, Term
import os
import unittest

devnull = open(os.devnull, 'w')
stdout = Term(devnull)

class TestTerm(unittest.TestCase):
    
    def set_display(self,codes=[], fg=None, bg=None):
        stdout.write(display(codes, fg, bg))

    def test_color(self):
        self.set_display('default')
        stdout.write("default text")
        
        stdout.write("regular foreground test:")
        for color in Magic.COLORS.keys():
            self.set_display(fg=color)
            stdout.write("    " + color)
        
        self.set_display('default')
        
        stdout.write("regular background test:")
        for color in Magic.COLORS.keys():
            self.set_display(bg=color)
            stdout.write("    " + color)
        
        self.set_display('default')
        
        stdout.write("bright foreground test:")
        self.set_display('bright')
        for color in Magic.COLORS.keys():
            self.set_display(fg=color)
            stdout.write("    " + color)
        
        self.set_display('default')
        
        stdout.write("dim foreground test:")
        self.set_display('dim')
        for color in Magic.COLORS.keys():
            self.set_display(fg=color)
            stdout.write("    " + color)
        
        self.set_display('bright','red')
        stdout.write("bright red")
        self.set_display("default")

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest
from fabulous import casts

class TestControl(unittest.TestCase):

    def test_casts_yes_no(self):
        self.failUnlessEqual(casts.yes_no('y'), True)
        self.failUnlessEqual(casts.yes_no('n'), False)
        self.failUnlessEqual(casts.yes_no('Yes'), True)
        self.failUnlessEqual(casts.yes_no('nO'), False)
        self.failUnlessRaises(ValueError, casts.yes_no, 'spam')
        
    def test_casts_file(self):
        self.failUnlessEqual( type(casts.file(__file__)), type(open(__file__)) )
        self.failUnlessRaises(ValueError, casts.file, '')
        

if __name__ == '__main__':
    unittest.main()
